#! python

import os
import sys
from subprocess import call
est_logging_file = "estserver.scripts.log"
openssl_cmd = "openssl"
prev_code = 0
windows = False
EST_OPENSSL_CACNF = "estExampleCA.cnf"
logandexit_msg = \
    """\
###########..EXIT..##########
SCRIPT $EST_SCRIPTNAME EXIT: %s (%d)
###########^^EXIT^^##########
 \
"""
headers_msg = \
    """\
Headers:
%s
/Headers\
"""

content_msg = \
    """\
Content:
%s
/Content\
"""


def logandexit(msg, code):
    with open(est_logging_file, "w") as file:
        file.write(logandexit_msg % (msg, code))
        sys.exit(code)


def iferrorlogandexit(msg, code):
    if prev_code:
        logandexit(msg, code)


def dumpheadersandcontent(header_file, content_file):
    with open(est_logging_file, "w") as file:
        if header_file:
            with open(header_file, "r") as headers:
                file.write(headers_msg % (headers.read()))
        if content_file:
            with open(content_file, "r") as content:
                file.write(content_msg % (content.read()))


def iferrorheaderslogandexit(search_hdr, hdr_file, log_msg, content_file):
    if not search_hdr in open(hdr_file).read:
        dumpheadersandcontent(hdr_file, content_file)
        logandexit("Header ERROR: %s" % log_msg, 1)


def cert2pkcs72stdout(crt_file):
    print("Content-Type: application/pkcs7-mime")
    print("")
    system("%s crl2pkcs7 -certfile %s -nocrl" %
           (openssl_cmd, crt_file))


def combinefiles(file1, file2, outfile):
    with open(outfile, "w") as fileout:
        with open(file1) as file:
            fileout.write(file.read())
    with open(outfile, "a") as fileout:
        with open(file2) as file:
            fileout.write(file.read())


def touch(fname, times=None):
    with open(fname, 'a'):
        os.utime(fname, times)


def system(cmd):
    if windows:
        cmd = cmd.replace('/', '\\')
    prev_code = call(cmd, shell=True)
    return prev_code


def detectWindowsFlag():
    global windows
    if (len(sys.argv) > 1):
        windows = (sys.argv[1] == "-w")



#!python
import shutil
from certgen import *
msg_start_warn = """\
#################################################################
SECURITY CONSIDERATIONS - NOTE WELL
The sample scripts used to handle EST operations are NOT
intended to provide a secure implementation. They have not
been evaluated for security, they have not had a Threat Model
reviewed, they are not particularly good about cleaning up after
themselves and they assume the data exchanged is well formed
if the cryptographic checks pass.

In short: They are not to be trusted. They provide a functional
implementation only.

Continuing will completely erase/destroy/nuke the existing 
TestMfg CSRs\
"""
msg_destroy_alert = """\
Nuking test CSRs!
#################################################################\
"""
msg_done_line = """\
#################################################################\
"""
testmfg_subj_line_additions = ['/serialNumber=SN:x, PID:x', '',
                               '/O=ToysRUsKnockOff',
                               '/OU=IOOT(Internet Of Other Things)',
                               '/uniqueMember=TheSecretCertClub']

detectWindowsFlag()
print(msg_start_warn)
try:
    input("Press Enter to continue...")
except:
    print("")
print(msg_destroy_alert)
for i in range(5):
    print('\"'+ "/CN=127.0.0.1" +
                          testmfg_subj_line_additions[i] + '\"')
for i in range(5):
    EST_OPENSSL_TESTMFGCADIR = "TestMfgCA%d" % (i)
    EST_OPENSSL_TESTMFGCACNF = "estExampleTestMfgCA.cnf"

    EST_TESTMFG_CERTANDKEY = "%s/private/certandkey1-esttestmfg%d.pem" % (
        EST_OPENSSL_TESTMFGCADIR, i)
    EST_TESTMFG_CCAUTHZDB = "ccAuthz.db"
    os.environ["MFGNUM"] = "%d" % (i)
    createCSR("TestMfgCA%d/test.csr" % (i), EST_TESTMFG_CERTANDKEY, "/CN=127.0.0.1" +
              testmfg_subj_line_additions[i], EST_OPENSSL_TESTMFGCACNF)

    print(msg_done_line)



#!python

import sys
import re
with open(sys.argv[1],'r') as logs:
    s = logs.read()
    matches = re.finditer(r'(?:\*\*\*EST \[TIMER\]\[)(\w+):(\d+)\]--> (.*) => (\d+\.\d+) seconds',s,)
    with open(sys.argv[2], 'w') as out_csv:
        out_csv.write("{},{},{},{}\n".format('Tag','Function','Line Number','Time(sec)'))
        for match in matches:
            (function_name, line_num, entry_tag, sec) = match.groups()
            out_csv.write("{},{},{},{}\n".format(entry_tag, function_name, line_num, sec))


#!python
import shutil
from certgen import *
msg_start_warn = """\
#################################################################
SECURITY CONSIDERATIONS - NOTE WELL
The sample scripts used to handle EST operations are NOT
intended to provide a secure implementation. They have not
been evaluated for security, they have not had a Threat Model
reviewed, they are not particularly good about cleaning up after
themselves and they assume the data exchanged is well formed
if the cryptographic checks pass.

In short: They are not to be trusted. They provide a functional
implementation only.

Continuing will completely erase/destroy/nuke the existing 
TestMfgCAs\
"""
msg_destroy_alert = """\
Nuking from orbit!
#################################################################\
"""

msg_create_CA = """\
#################################################################
####(Re)creating an initial root TestMfg%d CA certificate
#################################################################\
"""
msg_done_line = """\
#################################################################\
"""
msg_server_combine = """\
#################################################################
####Creating combined trusted cert file for server\
"""
msg_mfg_chain_combine = """\
#################################################################
####Creating combined trusted cert file for client\
"""
testmfg_subj_line_additions = ['/serialNumber=SN:x, PID:x', '',
                               '/O=ToysRUsKnockOff',
                               '/OU=IOOT(Internet Of Other Things)',
                               '/uniqueMember=TheSecretCertClub']

detectWindowsFlag()
print(msg_start_warn)
try:
    input("Press Enter to continue...")
except:
    print("")
print(msg_destroy_alert)
for i in range(5):
    print('\"'+ "/CN=127.0.0.1" +
                          testmfg_subj_line_additions[i] + '\"')
for i in range(5):
    EST_OPENSSL_TESTMFGCADIR = "TestMfgCA%d" % (i)
    EST_OPENSSL_TESTMFGCACNF = "estExampleTestMfgCA.cnf"
    EST_OPENSSL_INT_TESTMFGCACNF = "estExampleIntTestMfgCA.cnf"
    EST_OPENSSL_TESTMFGCACERT = "%s/cacert.crt" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSL_TESTMFGCAPRIVDIR = "%s/private" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSL_TESTMFGCANEWCERTSDIR = "%s/newcerts" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSL_TESTMFGCADATABASE = "%s/index.txt" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSL_TESTMFGCASERIAL = "%s/serial" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSL_TESTMFGCAPRIVKEY = "%s/cakey.pem" % EST_OPENSSL_TESTMFGCAPRIVDIR
    EST_OPENSSL_TESTMFGCAPRIVKEYPARAM = "%s/cakeyparam.pem" % EST_OPENSSL_TESTMFGCAPRIVDIR
    EST_OPENSSL_TESTMFGCASUBJ = "/CN=estExampleTestMfg%dCA" % (i)

    EST_OPENSSLCMD_TESTMFGCAECPARAMSFILE = "%s/prime256v1.pem" % EST_OPENSSL_TESTMFGCADIR
    EST_OPENSSLCMD_TESTMFGCANEWKEY_PARAM = "-newkey ec:%s" % EST_OPENSSLCMD_TESTMFGCAECPARAMSFILE

    EST_TESTMFG_SUBJ = "/CN=127.0.0.1"
    EST_TESTMFG_CERTREQ = "%s/esttestmfg%d.req" % (EST_OPENSSL_TESTMFGCADIR, i)
    EST_TESTMFG_CERT = "%s/esttestmfg%d.crt" % (EST_OPENSSL_TESTMFGCADIR, i)
    EST_TESTMFG_PRIVKEY = "esttestmfg%d.pem" % (i)
    EST_TESTMFG_CERTANDKEY = "%s/esttestmfg%d_certandkey.pem" % (
        EST_OPENSSL_TESTMFGCAPRIVDIR, i)
    EST_TESTMFG_CCAUTHZDB = "ccAuthz.db"
    os.environ["MFGNUM"] = "%d" % (i)

    shutil.rmtree(EST_OPENSSL_TESTMFGCADIR, ignore_errors=True)
    print(msg_create_CA)
    createCA(EST_OPENSSL_TESTMFGCASUBJ,
             EST_OPENSSL_TESTMFGCADIR,
             EST_OPENSSL_TESTMFGCACERT,
             EST_OPENSSL_TESTMFGCAPRIVDIR,
             EST_OPENSSL_TESTMFGCASERIAL,
             EST_OPENSSL_TESTMFGCADATABASE,
             EST_OPENSSL_TESTMFGCANEWCERTSDIR,
             EST_OPENSSL_TESTMFGCAPRIVKEY,
             EST_OPENSSLCMD_TESTMFGCANEWKEY_PARAM,
             EST_OPENSSLCMD_TESTMFGCAECPARAMSFILE)

    createIntermediateCAs(EST_OPENSSL_TESTMFGCADIR,
                          EST_OPENSSL_TESTMFGCACERT,
                          EST_OPENSSL_TESTMFGCAPRIVKEY,
                          EST_OPENSSL_TESTMFGCAPRIVKEYPARAM,
                          EST_OPENSSLCMD_TESTMFGCANEWKEY_PARAM,
                          '\"'+EST_TESTMFG_SUBJ +
                          testmfg_subj_line_additions[i] + '\"',
                          EST_OPENSSL_TESTMFGCACNF,
                          EST_OPENSSL_INT_TESTMFGCACNF,
                          EST_OPENSSL_TESTMFGCASUBJ,
                          EST_TESTMFG_PRIVKEY,
                          EST_TESTMFG_CERT,
                          "testmfg%d_chain.crt" % (i),
                          1,
                          1)

    print(msg_server_combine)
    with open("../trustedcerts.crt", "a") as fileout:
        with open("TestMfgCA%d/cacert.crt" % (i)) as filein:
            fileout.write(filein.read())

    print(msg_done_line)

    print(msg_mfg_chain_combine)
    combinefiles("../estCA/cacert.crt", "testmfg%d_chain.crt" % (i),
                 "trustedcertswithmfg%dchain.crt" % (i))
    print(msg_done_line)

print("Resetting the est server password file")
with open("../estCA/estpwdfile", "w") as file:
    file.write("estuser:estrealm:36807fa200741bb0e8fb04fcf08e2de6")
